use crate::db::Database;

use super::super::Result;

use super::super::error::Error as DiscordError;

use poise::serenity_prelude::{self as serenity, CreateEmbed, CreateMessage};
use tracing::{error, info};

fn create_embed_for_mention(
    message: &serenity::Message,
    guild_id: u64,
) -> CreateEmbed {
    let author_id = message.author.id.to_string();
    let message_channel = message.channel_id.get();
    CreateEmbed::new()
        .title("Someone mentioned everyone!")
        .field("Author", format!("<@{author_id}>"), true)
        .field("When", message.timestamp.naive_local().to_string(), true)
        .field("Channel", format!("<#{message_channel}>"), true)
        .field(
            "Link",
            format!(
                "https://discord.com/channels/{guild_id}/{message_channel}/{}",
                message.id
            ),
            false,
        )
}

/// Handle messages mentioning everyone.
///
/// # Errors
///
/// This function will return an error if a message fails to be sent,
/// if retrieving the list of channels registered as loggers fails, or
/// if there is not guild ID that can be retrieved from the message.
pub async fn handle_everyone_mention(
    ctx: &serenity::Context,
    database: &Database,
    message: &serenity::Message,
) -> Result {
    info!("Message mentioning everyone: {message:?}");
    if !message.mention_everyone {
        return Ok(());
    }
    if message.guild_id.is_none() {
        error!("Message without a guild_id! {message:?}");
        return Err(DiscordError::GuildIdNotFound.boxed());
    }
    let guild_id = message.guild_id.unwrap();
    let channels: Vec<serenity::ChannelId> =
        database.get_logging_channels(guild_id).await?;
    for channel in &channels {
        // Ignore result, it'll be in the bot's logger
        let embed = create_embed_for_mention(message, guild_id.get());
        let builder = CreateMessage::new().embed(embed);
        let _ = channel
            .send_message(&ctx, builder)
            .await
            .map_err(|e| error!("Failed to send message: {e:?}"));
    }
    Ok(())
}
